;; mew-win32.el  miscellaneous settings for Win32
;;
;;                     Written by Shuichi Kitaguchi <kit@Mew.org>

(defconst mew-win32-version "mew-win32.el v0.06")



(setq mew-prog-tar        "tar.exe")
(setq mew-prog-compress   "compress.exe")
(setq mew-prog-gzip       "gzip.exe")
(setq mew-prog-shell      shell-file-name)

(setq mew-prog-pgp        "pgp.exe")

(setq mew-prog-utime      "utime.exe")
(setq mew-touch-folder-p  t)


(cond
					; Meadow
 ((eq 20 emacs-major-version)
  (setq mew-prog-shell-arg  shell-command-switch)
  
  (make-coding-system 'no-conv 0 ?T "Do no conversion")
  (put 'no-conv 'eol-type 1)
  (setq mew-cs-noconv-eol 'no-conv-dos)
  
  (require 'win32-script)
  (define-process-argument-editing
    "/[^./]+$"
    'script-process-argument-editing-function))
					; Mule for Win32
 (t
  (setq mew-prog-shell-arg  shell-command-option)
  
  (make-coding-system
   '*crlf2lf* 4
   ?= "No conversion." nil
   '([1 23 47 346 13 18 250 10 18 1881 5 4421 19 10
	47 772 5892 49 58 0 0 14 772 49 22] .
	[1 4 14 775 22]))
  (setq mew-cs-noconv-eol *crlf2lf*)
  
  (require 'win32-script)
  (define-process-argument-editing
    "/[^./]+$"
    'script-process-argument-editing-function 'last)))


(defvar
  mew-mime-content-type
  (list
   (list 
    "multipart/*"
    nil
    nil
    nil
    (if (boundp 'mew-icon-multipart) mew-icon-multipart)
    )
   (list
    "audio/basic"
    "\\.au$"
    "base64"
    '("fiber.exe" () t)
    (if (boundp 'mew-icon-audio) mew-icon-audio)
    )
   (list
    "image/gif"
    "\\.gif$"
    "base64"
    '("fiber.exe" () t)
    (if (boundp 'mew-icon-image) mew-icon-image)
    )
   (list
    "image/jpeg"
    "\\.jpe?g$"
    "base64"
    '("fiber.exe" () t)
    (if (boundp 'mew-icon-image) mew-icon-image)
    )
   (list
    "image/x-xwd"
    "\\.xwd$"
    "base64"
    '("fiber.exe" () t)
    (if (boundp 'mew-icon-image) mew-icon-image)
    )
   (list
    "image/x-xbm"
    "\\.xbm$"
    "base64"
    '("fiber.exe" () t)
    (if (boundp 'mew-icon-image) mew-icon-image)
    )
   (list
    "image/x-bmp"
    "\\.bmp$"
    "base64"
    '("fiber.exe" () t)
    (if (boundp 'mew-icon-image) mew-icon-image)
    )
   (list
    "image/.*"
    "^$"
    "base64"
    '("fiber.exe" () t)
    (if (boundp 'mew-icon-image) mew-icon-image)
    )
   (list
    "video/mpeg"
    "\\.mpe?g$"
    "base64"
    '("fiber.exe" () t)
    (if (boundp 'mew-icon-video) mew-icon-video)
    )
   (list
    "application/postscript"
    "\\.ps$"
    "quoted-printable"
    '("fiber.exe" () t)
    (if (boundp 'mew-icon-application/postscript) 
	mew-icon-application/postscript)
    )
   (list
    "application/pgp-keys"
    "\\.pka$"
    nil
    '(mew-mime-pgp-keys () nil)
    (if (boundp 'mew-icon-unknown) mew-icon-unknown)
    )
   (list
    "application/octet-stream"
    "\\.[Ll][Zz][Hh]$"
    "base64"
    '("fiber.exe" () t)
    (if (boundp 'mew-icon-application/octet-stream)
	mew-icon-application/octet-stream)
    )
   (list
    "application/octet-stream"
    "\\.[Zz][Ii][Pp]$"
    "base64"
    '("fiber.exe" () nil)
    (if (boundp 'mew-icon-application/octet-stream)
	mew-icon-application/octet-stream)
    )
   (list
    "application/octet-stream"
    "\\.[Cc][Aa][Bb]$"
    "base64"
    '("fiber.exe" () nil)
    (if (boundp 'mew-icon-application/octet-stream)
	mew-icon-application/octet-stream)
    )
   (list
    "application/octet-stream"
    "\\.tar\\.?g?z?$"
    "base64"
    '("fiber.exe" () nil)
    (if (boundp 'mew-icon-application/octet-stream)
	mew-icon-application/octet-stream)
    )
   (list
    "application/octet-stream"
    "\\.[Tt][Gg][Zz]$"
    "base64"
    '("fiber.exe" () nil)
    (if (boundp 'mew-icon-application/octet-stream)
	mew-icon-application/octet-stream)
    )
   (list
    "application/octet-stream"
    "\\.gz$"
    "base64"
    '("fiber.exe" () nil)
    (if (boundp 'mew-icon-application/octet-stream)
	mew-icon-application/octet-stream)
    )
   (list
    "application/octet-stream"
    "\\.Z$"
    "base64"
    '("fiber.exe" () nil)
    (if (boundp 'mew-icon-application/octet-stream)
	mew-icon-application/octet-stream)
    )
   (list
    "application/octet-stream"
    "\\.pgp$"
    "base64"
    '(mew-mime-application/octet-stream () nil)
    (if (boundp 'mew-icon-application/octet-stream)
	mew-icon-application/octet-stream)
    )
   (list
    "message/external-body" 
    "\\.ext$"
    nil
    '(mew-mime-external-body () nil)
    (if (boundp 'mew-icon-message/external-body)
	mew-icon-message/external-body)
    )
   (list
    "message/rfc822"
    "^[0-9]+$"
    nil
    '(mew-mime-message/rfc822 () nil)
    (if (boundp 'mew-icon-message/rfc822) mew-icon-message/rfc822)
    )
   (list
    "message/delivery-status"
    "^$"
    nil
    '(mew-mime-text/plain () nil)
    (if (boundp 'mew-icon-text) mew-icon-text)
    )
   (list
    "text/richtext"
    "\\.rtf$"
    nil
    '(mew-mime-text/plain () nil)
    (if (boundp 'mew-icon-text) mew-icon-text)
    )
   (list
    "text/plain"
    ".*"
    nil
    '(mew-mime-text/plain () nil)
    (if (boundp 'mew-icon-text) mew-icon-text)
    )
   (list
    "text/.*"
    "^$"
    nil
    '(mew-mime-text/plain () nil)
    (if (boundp 'mew-icon-text) mew-icon-text)
    )
   (list
    ".*" 
    "^$"
    nil
    '(mew-mime-application/octet-stream () nil)
    (if (boundp 'mew-icon-unknown) mew-icon-unknown)
    )
   )
 "(content-type filename encoding (program (options ...) async) icon)"
 )


(provide 'mew-win32)
