;;;			     mew-func.el
;;;
;;;		Copyright (C) 1997  Kazuhiko Yamamoto
;;;
;;;		   This emacs lisp library conforms
;;;		GNU GENERAL PUBLIC LICENSE Version 2.
;;;
;;; Author:  Kazu Yamamoto <Kazu@Mew.org>
;;; Created: Mar 23, 1997
;;; Revised: Oct 25, 1997
;;;

(defconst mew-func-version "mew-func.el version 0.10")
(require 'mew)

;;
;; y-or-n
;;

(defvar mew-emacs-y-or-n-p-use nil
  "*SPC is y and RET is n if t. SPC is n and RET is y, otherwise.")

(defun mew-y-or-n-p (message &rest args)
  (interactive)
  (if args (setq message (apply (function format) message args)))
  (if mew-emacs-y-or-n-p-use
      (y-or-n-p message)
    (let ((char nil) (ociea cursor-in-echo-area))
      (unwind-protect
	  (progn
	    (setq cursor-in-echo-area t)
	    (catch 'break
	      (while t
		(message (concat message "(y=RET or n=SPC) "))
		(setq char (read-char))
		(cond
		 ((char-equal char ?y)
		  (setq char t)
		  (throw 'break nil))
		 ((char-equal char ?\r)
		  (setq char t)
		  (throw 'break nil))
		 ((char-equal char ?n)
		  (setq char nil)
		  (throw 'break nil))
		 ((char-equal char ?\ )
		  (setq char nil)
		  (throw 'break nil))
		 (t (ding))
		 )
		))
	    )
	(setq cursor-in-echo-area ociea)
	)
      char ;; return value
      ))
  )

;;
;; List functions
;;

(defun mew-member (a list)
  (catch 'member
    (while list
      (if (equal (car list) a)
	  (throw 'member t))
      (setq list (cdr list))
      ))
  )

(defun mew-folder-member (a list)
  (catch 'member
    (while list
      (if (or (equal (car list) a)
	      (equal (car list) (file-name-as-directory a)))
	  (throw 'member t))
      (setq list (cdr list))
      ))
  )

(defun mew-member-case-equal (str list)
  (let ((n 0))
    (catch 'member
      (while list
	(if (equal (downcase (car list)) (downcase str))
	    (throw 'member n))
	(setq list (cdr list))
	(setq n (1+ n))
	))
    ))

(defun mew-member-match (str list)
  (let ((n 0))
    (catch 'member
      (while list
	(if (string-match (car list) str)
	    (throw 'member n))
	(setq list (cdr list))
	(setq n (1+ n))
	))
    ))

;;
;; Associative list functions
;;

(defun mew-assoc-equal (key alist nth)
  (let (a n)
    (catch 'loop
      (while alist
	(setq a (car alist))
	(setq n (nth nth a))
	(if (or (equal n key) (eq n t))
	    (throw 'loop a))
	(setq alist (cdr alist))
	))
    ))

(defun mew-assoc-case-equal (key alist nth)
  (let ((skey (downcase key)) a n)
    (catch 'loop
      (while alist
	(setq a (car alist))
	(setq n (nth nth a))
	(if (or (and (stringp n) (equal (downcase n) skey))
		(eq n t))
	    (throw 'loop a))
	(setq alist (cdr alist))
	))
    ))

(defun mew-assoc-match (key alist nth)
  (let ((case-fold-search t) a n)
    (catch 'loop
      (while alist
	(setq a (car alist))
	(setq n (nth nth a))
	(if (or (and (stringp n) (string-match key n))
		(equal n key) (eq n t))
	    (throw 'loop a))
	(setq alist (cdr alist))
	))
    ))

(defun mew-assoc-match2 (key alist nth)
  (let ((case-fold-search t) a n)
    (catch 'loop
      (while alist
	(setq a (car alist))
	(setq n (nth nth a))
	(if (or (and (stringp n) (string-match n key))
		(equal n key) (eq n t))
	    (throw 'loop a))
	(setq alist (cdr alist))
	))
    ))

(defun mew-assoc-member (key lol nth)
  (let (l)
    (catch 'loop
      (while lol
	(setq l (car lol))
	(setq lol (cdr lol))
	(if (mew-member key (nth nth l)) (throw 'loop l))
	))
    ))
;;
;; Generic extract functions
;;

(defun mew-subsequence (seq beg &optional end)
  (cond
   ((vectorp seq)
    (mew-subvector seq beg end))
   ((stringp seq)
    (substring  seq beg end))
   ((listp seq)
    (mew-sublist seq beg end))
   (t nil)
   )
  )

(defun mew-sublist (list beg &optional end)
  (let (i ret)
    (if (null end) (setq end (length list)))
    (setq end (1- end))
    (setq i end)
    (while (<= beg i)
      (setq ret (cons (nth i list) ret))
      (setq i (1- i)))
    ret
    )
  )

(defun mew-subvector (vec beg &optional end)
  (let (i j ret)
    (if (null end) (setq end (length vec)))
    (setq ret (make-vector (- end beg) nil))
    (setq i beg j 0)
    (while (< i end)
      (aset ret j (aref vec i))
      (setq i (1+ i) j (1+ j)))
    ret
    )
  )

;;
;; String
;;

(defun mew-replace-character (string from to)
  (let ((len (length string))
	(cnt 0))
    (while (< cnt len)
      (if (char-equal (aref string cnt) from)
	  (aset string cnt to))
      (setq cnt (1+ cnt)))
    string
    ))

(defun mew-match (pos &optional string)
  "Substring POSth matched from STRING or the current buffer"
  (cond 
   ((stringp string)
    (substring string (match-beginning pos) (match-end pos)))
   (t 
    (mew-buffer-substring (match-beginning pos) (match-end pos)))
   )
  )

;;
;; Stolen form Perl
;;

(defun mew-join (separator string-list)
  (let ((ret-str ""))
    (if (null string-list) 
	"" ;; return strings anyway
      (setq ret-str (car string-list))
      (setq string-list (cdr string-list))
      (while string-list
	(setq ret-str (concat ret-str separator (car string-list)))
	(setq string-list (cdr string-list))
	)
      ret-str
      )
    ))

(defun mew-split (separator str)
  (let ((len (length separator))
	(reg-sep (regexp-quote separator))
	(start 0)
	(last 0)
	(ret-list ()))
    (while (setq start (string-match reg-sep str start))
      (setq ret-list (cons (substring str last start) ret-list))
      (setq start (+ start len))
      (setq last start)
      )
    (setq ret-list (cons (substring str last nil) ret-list))
    (nreverse ret-list)
    ))

;;
;; Folder
;;

;(defun mew-expand-file-name (msg &optional buffer-or-name)
;  (if (bufferp buffer-or-name)
;      (setq buffer-or-name (buffer-name buffer-or-name)))
;  (and buffer-or-name 
;       (setq buffer-or-name (directory-file-name buffer-or-name)))
;  (if buffer-or-name
;      (expand-file-name 
;       (expand-file-name (mew-folder-to-dir msg)
;			 (mew-folder-to-dir buffer-or-name))
;       mew-mail-path)
;    (expand-file-name (mew-folder-to-dir msg) mew-mail-path))
;  )
;

(defun mew-folder-mailp (folder)
  (string-match "^+" folder)
  )

(defun mew-folder-newsp (folder)
  (string-match "^-" folder)
  )

(defun mew-folder-local-newsp (folder)
  (string-match "^=" folder)
  )

(defun mew-folder-to-dir (folder)
  (if (string-match "^[+=]" folder)
      (substring folder 1 nil)
    folder)
  )

(defun mew-expand-folder (folder &optional message)
  (let ((prefix-char (aref folder 0))
	(subdir (substring folder 1 nil))
	dir)
    (cond
     ((char-equal prefix-char ?+)
      (setq dir (expand-file-name subdir mew-mail-path)))
     ((char-equal prefix-char ?=)
      (setq dir (expand-file-name subdir mew-news-path)))
     )
    (if message
	(expand-file-name message dir)
      dir)
    ))

(defun mew-folder-check (folder)
  "A function to see if FOLDER exists.
Return t if exists or created. Otherwise, return nil."
  (if (not (stringp folder))
      nil ;; wrong
    (let ((absdir (mew-expand-folder folder)))  ;; /home/Mail/foo
      (if (file-exists-p absdir)
	  (if (file-directory-p absdir)
	      t ;; exists
	    (message "%s is a file" folder)
	    nil ;; exists but a file
	    )
	(if (not (mew-y-or-n-p "%s does not exist. Create it? " folder))
	    nil ;; not created
	  (mew-make-directory absdir)
	  (setq mew-folder-list (cons folder mew-folder-list))
	  (setq mew-folder-alist
		(cons (mew-folder-pair folder) mew-folder-alist))
	  (if (file-readable-p mew-folders-file)
	      (save-excursion
		(mew-set-buffer-tmp)
		(insert folder)
		(insert "\n")
		(append-to-file (point-min) (point-max) mew-folders-file)
		)
	    )
	  t ;; created
	  )))))

(defun mew-folder-new-message (folder &optional num-only)
  (save-excursion
    (mew-set-buffer-tmp)
    (call-process mew-prog-impath nil (current-buffer) nil folder "new")
    (goto-char (point-min))
    (if (re-search-forward "\\([0-9]+\\)$" nil t)
	(if num-only
	    (mew-match 1)
	  (beginning-of-line)
	  (let ((start (point)))
	    (forward-line)
	    (mew-buffer-substring start (1- (point))))))))

;;
;; Directory
;;

(defmacro mew-directory-empty-p (dir)
  "See if DIR never contains files"
  (` (null (car (cdr (cdr (directory-files (, dir)))))))
  )

(defun mew-make-directory (path)
  (let ((parent (directory-file-name (file-name-directory path))))
    (if (null (file-directory-p parent))
	(mew-make-directory parent))
    (if (and (file-exists-p path) (not (file-directory-p path)))
	(delete-file path))
    (make-directory path)
    ))

(defun mew-delete-directory-recursively (dir)
  (let ((files (directory-files dir t "^[^.]\\|^.[^.]")))
    (while files
      (cond
       ((file-directory-p (car files))
	(mew-delete-directory-recursively (car files)))
       (t
	(delete-file (car files)))
       )
      (setq files (cdr files))
      ))
  (delete-directory dir)
  )

;;
;; File existence
;;

(defun mew-which (file path)
  (catch 'loop
    (while path
      (if (file-exists-p (expand-file-name file (car path)))
	  (throw 'loop (expand-file-name file (car path)))
	(setq path (cdr path)))
      )
    ))

(defun mew-which-el (elfile path)
  (or (mew-which (concat elfile ".el") path)
      (mew-which (concat elfile ".elc") path))
  )

;;
;; Random
;;

(defvar mew-random-last nil)

(defun mew-random-string ()
  (let ((num (% (abs (random)) 1000)))
    (while (equal mew-random-last num)
      (setq num (% (abs (random)) 1000))
      )
    (setq mew-random-last num)
    (int-to-string num)
    ))

(defvar mew-random-base "0123456789abcdefghijklmnopqrstuvwxwz")

(defun mew-random-char ()
  (let* ((len (length mew-random-base))
	 (idx (% (abs (random)) len)))
    (substring mew-random-base idx (1+ idx))
    ))

(defun mew-random-filename (dir &optional suffix)
  (let ((cnt 0) (max (length mew-random-base))
	file filepath)
    (setq file (concat (mew-random-char) suffix))
    (setq filepath (expand-file-name file dir))
    (while (and (file-exists-p filepath) (< cnt max))
      (setq file (concat (mew-random-char) suffix))
      (setq filepath (expand-file-name file dir))
      (setq cnt (1+ cnt)))
    (if (file-exists-p filepath) 
	nil
      filepath)
    ))

;;
;; Buffer
;;

(defun mew-set-buffer-tmp (&optional dir)
  (set-buffer (get-buffer-create mew-buffer-tmp))
  (mew-erase-buffer)
  (if dir (cd dir) (cd mew-home))
  )

(defun mew-erase-buffer ()  
  (setq buffer-read-only nil)
  (widen)
  (erase-buffer)
  )

(provide 'mew-func)
